<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "customer".
 *
 * @property integer $id
 * @property string $username
 * @property string $password_hash
 * @property string $name
 * @property string $surname
 * @property string $phone
 * @property string $date_time
 * @property string $gender
 * @property integer $birth_day
 * @property integer $birth_month
 * @property integer $birth_year
 * @property string $body
 * @property integer $group_id
 * @property string $email
 * @property string $auth_key
 * @property string $password_reset_token
 * @property integer $status
 * @property integer $created_at
 * @property integer $updated_at
 *
 * @property ArtboxComment[] $artboxComments
 * @property ArtboxCommentRating[] $artboxCommentRatings
 * @property ArtboxLike[] $artboxLikes
 * @property Share[] $shares
 */
class Customer extends User implements \yii\web\IdentityInterface
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'customer';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['username', 'password_hash'], 'required'],
            [['date_time'], 'safe'],
            [['birth_day', 'birth_month', 'birth_year', 'group_id', 'status', 'created_at', 'updated_at'], 'integer'],
            [['body'], 'string'],
            [['status'],'default', 'value' => '10'],
            [['username', 'name', 'surname', 'phone', 'email', 'password_reset_token'], 'string', 'max' => 255],
            [['gender', 'auth_key'], 'string', 'max' => 32],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'username' => 'Username',
            'name' => 'Name',
            'surname' => 'Surname',
            'phone' => 'Phone',
            'date_time' => 'Date Time',
            'gender' => 'Gender',
            'birth_day' => 'Birth Day',
            'birth_month' => 'Birth Month',
            'birth_year' => 'Birth Year',
            'body' => 'Body',
            'group_id' => 'Group ID',
            'email' => 'Email',
            'auth_key' => 'Auth Key',
            'password_reset_token' => 'Password Reset Token',
            'status' => 'Status',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShares()
    {
        return $this->hasMany(Share::className(), ['user_id' => 'id']);
    }

    /**
     * Finds user by email
     *
     * @param string $email
     * @return static|null
     */
    public static function findByEmail($email)
    {
        return static::findOne(['email' => $email, 'status' => self::STATUS_ACTIVE]);
    }

    public function getName(){
        return $this->username. ' '.$this->surname;
    }


}
