<?php

namespace common\models;
use common\behaviors\SaveImgBehavior;
use common\modules\product\models\Product;
use Yii;
use yii\behaviors\TimestampBehavior;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "event".
 *
 * @property integer $event_id
 * @property string $name
 * @property string $alias
 * @property string $body
 * @property string $image
 * @property string $meta_title
 * @property string $description
 * @property string $h1
 * @property string $seo_text
 * @property integer $created_at
 * @property integer $updated_at
 * @property integer $end_at
 * @property integer $status
 * @property integer $banner
 */
class Event extends \yii\db\ActiveRecord
{
    public $imageUpload;
    public $products_file;

    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'event';
    }




    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'img'=>[
                'class'     => SaveImgBehavior::className(),
                'fields' => [
                    ['name'=>'imageUpload','directory' => 'event' ],
                    ['name'=>'banner','directory' => 'event' ],
                ]
            ],
            TimestampBehavior::className(),
            'slug' => [
                'class' => 'common\behaviors\Slug',
                'in_attribute' => 'name',
                'out_attribute' => 'alias',
                'translit' => true
            ],
            [
                'class'              => 'common\behaviors\ShowImage',
            ],
        ];
    }


    public function beforeSave($insert)
    {
        if (parent::beforeSave($insert)) {

            $this->end_at = !empty($this->end_at) ? strtotime($this->end_at) : '';
            return true;
        } else {
            return false;
        }
    }

    public function afterFind(){
        $this->end_at = !empty($this->end_at) ? date("Y-m-d", $this->end_at) : '';
    }


    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['body', 'seo_text'], 'string'],
            [['created_at', 'updated_at','percent' ], 'integer'],
            [['name', 'alias', 'image', 'meta_title', 'description', 'h1','end_at','banner'], 'string', 'max' => 255],
            [['name','body'], 'required'],
            [['imageUpload','sale','is_event'], 'safe'],
            [['imageUpload'], 'file', 'extensions' => 'jpg, gif, png'],
            [['products_file'], 'file'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'event_id' => Yii::t('app', 'ID акции'),
            'name' => Yii::t('app', 'name'),
            'alias' => Yii::t('app', 'alias'),
            'body' => Yii::t('app', 'body'),
            'image' => Yii::t('app', 'image'),
            'meta_title' => Yii::t('app', 'meta_title'),
            'description' => Yii::t('app', 'description'),
            'h1' => Yii::t('app', 'h1'),
            'seo_text' => Yii::t('app', 'seo_text'),
            'created_at' => Yii::t('app', 'created_at'),
            'updated_at' => Yii::t('app', 'updated_at'),
            'end_at' => Yii::t('app', 'end_at'),
            'status' => Yii::t('app', 'Статус акции'),
            'products_file' => Yii::t('app', 'Загрузка файла'),
            'sale' => Yii::t('app', 'Распродажа'),
            'percent' => Yii::t('app', 'Процент'),
            'is_event' => Yii::t('app', 'Акция'),
        ];
    }


    public function isActive(){
        if($this->status){

            if(!empty($this->end_at) && (strtotime($this->end_at) <= strtotime(date("Y-m-d")))){
                return false;
            }
            return true;
        }
        return false;
    }



    public function getProducts(){
        return $this->hasMany(Product::className(),['product_id' => 'product_id'] )->viaTable('events_to_products', ['event_id' => 'event_id']);
    }

    public static function getSaleEvents(){
        return ArrayHelper::toArray(self::find()->select('percent')->distinct('percent')->where('sale=true AND percent IS NOT NULL')->orderBy('percent')->all());
    }


}
