<?php

namespace common\modules\product\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;

/**
 * ProductVariantSearch represents the model behind the search form about `backend\models\ProductVariant`.
 */
class ProductVariantListSearch extends ProductVariant
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['product_variant_id', 'product_id', 'product_unit_id', 'product_variant_type_id'], 'integer'],
            [['name', 'sku', 'remote_id'], 'safe'],
            [['price', 'price_old', 'stock'], 'number'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params, $product_id = false)
    {
        $query = ProductVariant::find();

        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        if($product_id){
            $query->andFilterWhere([
                'product_id' => $product_id,
            ]);
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'product_variant_id' => $this->product_variant_id,
            'price' => $this->price,
            'price_old' => $this->price_old,
            'stock' => $this->stock,
            'product_unit_id' => $this->product_unit_id,
            'product_variant_type_id' => $this->product_variant_type_id,
        ]);

        $query->andFilterWhere(['like', 'name', $this->name])
            ->andFilterWhere(['like', 'sku', $this->sku])
            ->andFilterWhere(['like', 'remote_id', $this->remote_id]);

        return $dataProvider;
    }
}
