<?php

namespace common\modules\product\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use yii\web\NotFoundHttpException;

/**
 * ProductVariantSearch represents the model behind the search form about `common\modules\product\models\ProductVariant`.
 */
class ProductVariantSearch extends ProductVariant
{
    public $brand_name;
    public $category_name;
    public $tax_group_id;
    public $is_top;
    public $is_new;
    public $akciya;
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['name', 'fullname', 'sku', 'price', 'price_old', 'stock', 'brand_name', 'category_name'], 'safe'],
            [['product_variant_id', 'product_id'], 'integer'],
            [['is_top', 'is_new', 'akciya'], 'boolean'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = ProductVariant::find();

        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);

        $this->load($params);

        if (!empty($params['product_id'])) {
            $this->product_id = $params['product_id'];
        }

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        $dataProvider->setSort([
            'attributes' => [
                'name' => [
                    'asc' => ['product_variant.name' => SORT_ASC],
                    'desc' => ['product_variant.value' => SORT_DESC],
                    'default' => SORT_DESC,
                    'label' => 'Variant name',
                ],
                'brand_name' => [
                    'asc' => ['brand_name.value' => SORT_ASC],
                    'desc' => ['brand_name.value' => SORT_DESC],
                    'default' => SORT_DESC,
                    'label' => 'Brand name',
                ],
                'category_name',
                'sku',
            ]
        ]);

        $query->joinWith(['product', 'product.brand', 'product.categories']);

        if (isset($this->is_top)) {
            $query->andFilterWhere([
                'product.is_top' => (bool)$this->is_top,
            ]);
        }
        if (isset($this->is_new)) {
            $query->andFilterWhere([
                'product.is_new' => (bool)$this->is_new,
            ]);
        }
        if (isset($this->akciya)) {
            $query->andFilterWhere([
                'product.akciya' => (bool)$this->akciya,
            ]);
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'product_variant.product_id' => $this->product_id,
            'product_variant_id' => $this->product_variant_id,
        ]);

        if (!empty($this->fullname)) {
            $query->orFilterWhere(['like', 'product_variant.name', $this->fullname]);
            $query->orFilterWhere(['ilike', 'product.name', $this->fullname]);
        }
        $query->andFilterWhere(['ilike', 'product.brand_name.value', $this->brand_name]);
        $query->andFilterWhere(['ilike', 'product.category_name.value', $this->category_name]);
        $query->andFilterWhere(['ilike', 'sku', $this->sku]);

        $query->groupBy(['product_variant_id']);
        $query->orderBy('product_variant.product_id', 'ASC');

        return $dataProvider;
    }

    public static function findBySku($sku) {
        /** @var ProductQuery $query */
        $query = ProductVariant::find()
                ->andFilterWhere(['sku' => $sku]);
        if (($model = $query->one()) !== null) {
            return $model;
        }
        return;
    }

    public static function findByRemoteID($id) {
        /** @var CategoryQuery $query */
        $query = ProductVariant::find()
            ->andFilterWhere(['remote_id' => $id]);
        if (($model = $query->one()) !== null) {
            return $model;
        }
        return;
    }
}
