<?php

use yii\helpers\Html;
use yii\grid\GridView;
use yii\helpers\Url;

/* @var $this yii\web\View */
/* @var $searchModel common\modules\product\models\ProductVariantSearch */
/* @var $dataProvider yii\data\ActiveDataProvider */

$this->title = Yii::t('product', 'Variants');
$this->params['breadcrumbs'][] = ['label' => Yii::t('product', 'Products'), 'url' => ['/product/manage']];
if (!empty($product)) {
    $this->params['breadcrumbs'] = [
        ['label' => Yii::t('product', 'Variants'), 'url' => ['/product/variant']],
        $product->fullname
    ];
} else {
    $this->params['breadcrumbs'][] = $this->title;
}
?>
<div class="product-index">

    <h1><?= Html::encode($this->title) ?></h1>
    <?php // echo $this->render('_search', ['model' => $searchModel]); ?>

    <p>
        <?= Html::a(Yii::t('product', 'Create Variant'), Url::toRoute(['create','product_id'=> $product_id]), ['class' => 'btn btn-success']) ?>
    </p>
    <?= GridView::widget([
        'dataProvider' => $dataProvider,
        'filterModel' => $searchModel,
        'columns' => [
            ['class' => 'yii\grid\SerialColumn'],

            [
                'attribute' => 'product_id',
                'value' => 'fullname',
                'label' => Yii::t('product', 'Name'),
                'filter' => \kartik\select2\Select2::widget([
                    'model' => $searchModel,
                    'attribute' => 'product_id',
                    'data' => \yii\helpers\ArrayHelper::map(\common\modules\product\models\Product::find()->orderBy(['name' => 'ASC'])->all(), 'product_id', 'name'),
                    'language' => 'ru',
                    'options' => [
                        'placeholder' => Yii::t('product', 'Select product'),
                        'multiple' => false,
                    ],
                    'pluginOptions' => [
                        'allowClear' => true
                    ],
                ]),
            ],
            'sku',
            'price',
            'price_old',
            'stock',

            [
                'class'          => 'yii\grid\ActionColumn',
                'buttons'        => [
                    'view' => function ($url, $model)
                    {
                        return Html::a (
                            '<span class="glyphicon glyphicon-eye-open"></span>',
                            Url::to(['view','product_id'=> $model->product_id, 'id' => $model->product_variant_id]),
                            [
                                'title' => "Просмотр",
                            ]
                        );
                    },
                    'update' => function ($url, $model)
                    {
                        return Html::a (
                            '<span class="glyphicon glyphicon-pencil"></span>',
                            Url::to(['update','product_id'=> $model->product_id, 'id' => $model->product_variant_id]),
                            [
                                'title' => "Редактировать",
                            ]
                        );
                    },
                    'delete' => function ($url, $model)
                    {

                        return Html::a('<span class="glyphicon glyphicon-trash"></span>',  Url::to(['delete','product_id'=> $model->product_id, 'id' => $model->product_variant_id]), [
                            'title' => Yii::t('yii', 'Delete'),
                            'data-confirm' => Yii::t('yii', 'Are you sure to delete this item?'),
                            'data-method' => 'post',
                        ]);

                    },
                ],
            ],
        ],
    ]); ?>
</div>
