<?php

namespace common\modules\rubrication\controllers;

use common\modules\rubrication\models\TaxOption;
use Yii;
use common\modules\rubrication\models\TaxGroup;
use yii\data\ActiveDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * TaxGroupController implements the CRUD actions for TaxGroup model.
 */
class TaxGroupController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all TaxGroup models.
     * @param $level integer
     * @return mixed
     */
    public function actionIndex($level)
    {
        $dataProvider = new ActiveDataProvider([
            'query' => TaxGroup::find()->where(['level' => $level]),
        ]);

        return $this->render('index', [
            'dataProvider' => $dataProvider,
            'level' => $level
        ]);
    }

    /**
     * Displays a single TaxGroup model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($level,$id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new TaxGroup model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @param $level integer
     * @return mixed
     */
    public function actionCreate($level)
    {
        $model = new TaxGroup();

        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            $model->level  = $level;
            $model->save();
            return $this->redirect(['index', 'level' => $level]);
        } else {
            return $this->render('create', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Updates an existing TaxGroup model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param $level integer
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($level,$id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['index', 'level' => $level]);
        } else {
            return $this->render('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing TaxGroup model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param $level integer
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($level,$id)
    {
        $this->findModel($id)->delete();
        return $this->redirect(['index', 'level' => $level]);
    }

    /*
     * Group-relations
     */
    public function actionRelation($id, $relations = ['tax_option_to_group', 'tax_option_to_option']) {
        $group = $this->findModel($id);
        $items = [];

        foreach($relations as $relation) {
            $rows = $group->getRelations($relation)->all();
            $items = array_merge($items, $rows);
        }

        return $this->render('relations', [
            'items' => $items,
            'group' => $group,
        ]);
    }

    /*
     * Rebuilp MP-params for group options
     */
    public function actionRebuild($id) {
        TaxOption::find()->rebuildMP($id);

        return $this->redirect(['index']);
    }

    /**
     * Finds the TaxGroup model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return TaxGroup the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = TaxGroup::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
}
